"""
Python module for Hiera writing
"""
import yaml
import pwd
import os

class HieraWrite:
    """
    Class for writing values to hiera files
    """

    def hiera_paths(self, yaml_path='/etc/puppet/hieradata/',
                    userdata='/etc/puppet/userdata/'):
        """
        Define hiera paths, the default paths
        are based on /etc/locmap/hiera.yaml.
        If hiera.yaml needs to change
        you can define the new paths with this method.
        """
        self.yaml_path = yaml_path
        self.userdata = userdata
        return self


    def local_users(self):
        """
        Find the local afs users if they exist.
        """
        local_users = []
        local_users.append('root')
        for entry in pwd.getpwall():
            if 'afs' in entry.pw_dir:
                local_users.append(entry.pw_name)
        yamlfile = self.yaml_path+"module_names/sendmail/sendmail.yaml"
        if not os.path.exists(os.path.dirname(yamlfile)):
            try:
                os.makedirs(os.path.dirname(yamlfile))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    raise
        with open(yamlfile, 'w') as sendmailfile:
            mydata = {'local_users':local_users}
            sendmailfile.write(yaml.dump(mydata, explicit_start=True,
                                         default_flow_style=False))

    def sudo_users(self, node, suser):
        """
        Give the responsible user or egroup users sudo privileges.
        """
        yamlfile = self.yaml_path+"module_names/sudo/sudo.yaml"
        if not os.path.exists(os.path.dirname(yamlfile)):
            try:
                os.makedirs(os.path.dirname(yamlfile))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    raise
        if node == 'egroup':
            with open(yamlfile, 'w') as sudofile:
                mydata = {'sudo_users':suser}
                sudofile.write(yaml.dump(mydata, explicit_start=True,
                                         default_flow_style=False))
        elif node == 'user':
            sudo_users = []
            sudo_users.append(suser)
            with open(yamlfile, 'w') as sudofile:
                mydata = {'sudo_users':sudo_users}
                sudofile.write(yaml.dump(mydata, explicit_start=True,
                                         default_flow_style=False))

    def rootk5login(self, node, k5user):
        """
        Add accounts to /root/.k5login file.
        """
        k5entries = []
        yamlfile = self.yaml_path+"module_names/kerberos/kerberos.yaml"
        if not os.path.exists(os.path.dirname(yamlfile)):
            try:
                os.makedirs(os.path.dirname(yamlfile))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    raise
        if node == 'egroup':
            for user in k5user:
                k5entries.append(user+"@CERN.CH")
            with open(yamlfile, 'w') as kerberosfile:
                mydata = {'k5login_values':k5entries}
                kerberosfile.write(yaml.dump(mydata, explicit_start=True,
                                             default_flow_style=False))
        elif node == 'user':
            k5entries.append(k5user+"@CERN.CH")
            with open(yamlfile, 'w') as kerberosfile:
                mydata = {'k5login_values':k5entries}
                kerberosfile.write(yaml.dump(mydata, explicit_start=True,
                                             default_flow_style=False))

    def sshkeys_hiera(self, sshusers, sshkeys):
        """
        Write the sshkeys of responsible user(s).
        """
        yamlfile = self.yaml_path+"module_names/ssh/ssh.yaml"
        if not os.path.exists(os.path.dirname(yamlfile)):
            try:
                os.makedirs(os.path.dirname(yamlfile))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    raise
        if sshusers:
            print '[INFO] User(s) %s found with public ssh key.\n' % sshusers
        with open(yamlfile, 'w') as sshfile:
            if sshkeys:
                mydata = {'foundkey':True}
                sshfile.write(yaml.dump(mydata, explicit_start=True,
                                        default_flow_style=False))
                mydata = {'pubkeys':sshkeys}
                sshfile.write(yaml.dump(mydata, default_flow_style=False))

    def rootfw_hiera(self, mails,):
        """
        Write the responsible user(s) mail.
        """
        yamlfile = self.yaml_path+"module_names/sendmail/sendmail.yaml"
        if not os.path.exists(os.path.dirname(yamlfile)):
            try:
                os.makedirs(os.path.dirname(yamlfile))
            except OSError as exc:
                if exc.errno != errno.EEXIST:
                    raise
        with open(yamlfile, 'a') as sendmail:
            mydata = {'mails':mails}
            sendmail.write(yaml.dump(mydata, default_flow_style=False))

    def lpadmin_hiera(self, buildings):
        """
        Write the appropriate building(s) to hiera.
        """
        if buildings:
            yamlfile = self.yaml_path+"module_names/lpadmin/lpadmin.yaml"
            if not os.path.exists(os.path.dirname(yamlfile)):
                try:
                    os.makedirs(os.path.dirname(yamlfile))
                except OSError as exc:
                    if exc.errno != errno.EEXIST:
                        raise
            with open(yamlfile, 'w') as lpfile:
                mydata = {'lpadmin::buildings':buildings}
                lpfile.write(yaml.dump(mydata, explicit_start=True,
                                       default_flow_style=False))

    def node_info(self, hostname):
        """
        Extract all the useful info from xldapclient class
        to inject them in hiera functions above.
        """
        import locmap.xldapclient

        config = locmap.xldapclient.XldapClient()
        config.cn(hostname)
        if config.node == 'egroup':
            node = 'egroup'
            suser = config.egroup_users
            k5user = config.egroup_users
            respuser = config.egroup_users
            mainuser = None
        elif config.node == 'user':
            node = 'user'
            suser = config.respuser
            mainuser = config.mainuser
            k5user = config.respuser
            respuser = config.respuser
        return node, suser, k5user, respuser, mainuser
