import ldap
import ldap.sasl
import re

class XldapClient:
    """
    XldapCLient class is responsible for interacting with xldap.cern.ch
    """
    def __init__(self):
        """
        Lookup xldap.cern.ch
        """
        try:
            self.server = "xldap.cern.ch"
            self.connection = ldap.open(self.server)
            self.searchScope = ldap.SCOPE_SUBTREE
            self.baseDN = "OU=Users,OU=Organic Units,DC=cern,DC=ch"
        except ldap.LDAPError, error_message:
            print error_message

    def cern_egroups_expansion(self, egroups, conn,
                               processed, recursion=True, fmt="%s"):
        """ Expands a list of egroups.

    Arguments:
    egroups -- A list of egroups to expand. ['ai-admins', 'it-dep']
    conn -- An initialized LDAP connection object
    processed -- A list of already processed egroups. Pass [].
    recursion -- Boolean to enable recursion
    fmt -- Format string to print usernames. Must have '%s'

    Returns:
    A set of formatted usernames (empty if all egroups are empty)
        """
        base = 'OU=e-groups,OU=Workgroups,DC=cern,DC=ch'
        user_regexp = r'CN=(\S+),OU=Users,OU=Organic Units,DC=cern,DC=ch'
        egroup_regexp = r'CN=(\S+),OU=e-groups,OU=Workgroups,DC=cern,DC=ch'

        users = set()
        for egroup in egroups:
            if egroup in processed:
                continue
            processed.append(egroup)
            egroup_filter = "(&(objectClass=group)(CN=%s))" % egroup
            ldap_results = conn.search_s(base, ldap.SCOPE_SUBTREE,
                                         egroup_filter, ['member'])
            for dn, entry in ldap_results:
                if 'member' in entry:
                    for result in entry['member']:
                        match = re.match(user_regexp, result)
                        if match is not None:
                            users.add(fmt % match.group(1))
                            continue
                        if recursion == True:
                            match = re.match(egroup_regexp, result)
                            if match is not None:
                                expanded_egroup = self.cern_egroups_expansion([match.group(1)],
                                                                              conn, processed, recursion, fmt)
                                users = users.union(expanded_egroup)

        return users



    def cn(self, hostname):
        """
        Grab the responsible and main user/egroup cn from xldap
        """
        try:
            baseDN = "OU=CERN Linux Computers,DC=cern,DC=ch"
            retrieveAttributes = ['managedby', 'manager']
            searchFilter = "(&(objectClass=computer)(|(cn=%s)))" % hostname
            ldap_result_id = self.connection.search(baseDN, self.searchScope,
                                                    searchFilter, retrieveAttributes)
            result_type, result_data = self.connection.result(ldap_result_id, 0)
            if not result_data:
                print "[INFO] Hostname '%s' not in %s .\n" % (hostname, baseDN)
                baseDN = "DC=cern,DC=ch"
                searchFilter = "(&(objectClass=computer)(|(cn=%s)))" % hostname
                ldap_result_id = self.connection.search(baseDN,
                                                        self.searchScope,
                                                        searchFilter,
                                                        retrieveAttributes)
                result_type, result_data = self.connection.result(ldap_result_id, 0)
            if result_data:
                # Find if responsible/manager is an egroup or a user.
                id_node = result_data[0][1]['managedBy'][0].split(",")[1]
                id_manager_node = result_data[0][1]['manager'][0].split(",")[1]
                # For responsible egroup/user
                if id_node == 'OU=e-groups':
                    XldapClient.node = 'egroup'
                    egroup = result_data[0][1]['managedBy'][0].split(",")[0].strip("CN=")
                    conn = ldap.initialize("ldap://%s" % self.server)
                    users = self.cern_egroups_expansion([egroup], conn, [])
                    conn.unbind()
                    XldapClient.egroup_users = sorted(list(users))
                elif id_node == 'OU=Users':
                    XldapClient.node = 'user'
                    XldapClient.respuser = result_data[0][1]['managedBy'][0].split(",")[0].strip("CN=")
                # For egroup/user manager
                if id_manager_node == 'OU=e-groups':
                    XldapClient.manager_node = 'egroup'
                    manager_egroup = result_data[0][1]['manager'][0].split(",")[0].strip("CN=")
                    conn = ldap.initialize("ldap://%s" % self.server)
                    manager_users = self.cern_egroups_expansion([manager_egroup], conn, [])
                    conn.unbind()
                    XldapClient.manager_egroup_users = sorted(list(manager_users))
                elif id_manager_node == 'OU=Users':
                    XldapClient.manager_node = 'user'
                    XldapClient.mainuser = result_data[0][1]['manager'][0].split(",")[0].strip("CN=")

            else:
                print "[INFO] Hostname '%s' not in %s .\n" % (hostname, baseDN)
        except ldap.LDAPError, error_message:
            print error_message

    def sshkeys(self):
        """
        Grab the first ssh key of the responsible user or egroup users
        """
        try:
            retrieveAttributes = ['altSecurityIdentities']
            if XldapClient.node == 'egroup':
                sshkeyid = []
                usersfound = []
                for user in XldapClient.egroup_users:
                    searchFilter = "(&(objectClass=user)(|(cn=%s)))" % user
                    ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                    result_type, result_data = self.connection.result(ldap_result_id, 0)
                    altlist = sorted(result_data[0][1]['altSecurityIdentities'])
                    for entry in altlist:
                        if 'SSH:' in entry:
                            sshkeyid.append(entry.strip('SSH:'))
                            usersfound.append(user)
                sshusers = list(set(usersfound))
                return sshusers, sshkeyid
            elif XldapClient.node == 'user':
                # We have a user node machine
                searchFilter = "(&(objectClass=user)(|(cn=%s)))" % XldapClient.respuser
                ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                result_type, result_data = self.connection.result(ldap_result_id, 0)
                altlist = sorted(result_data[0][1]['altSecurityIdentities'])
                usersfound = []
                sshkey = []
                for entry in altlist:
                    if 'SSH:' in entry:
                        sshkey.append(entry.strip('SSH:'))
                        usersfound.append(XldapClient.respuser)
                sshusers = list(set(usersfound))
                return sshusers, sshkey
        except AttributeError, error_message:
            pass


    def rootforward(self):
        """
        Return the mail of the responsible user of a machine in
        order to write in /root/.forward
        """
        try:
            retrieveAttributes = ['mail']
            if XldapClient.node == 'user':
                mails = []
                users = []
                searchFilter = "(&(objectClass=user)(|(cn=%s)))" % XldapClient.respuser
                ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                result_type, result_data = self.connection.result(ldap_result_id, 0)
                if result_data:
                    mail = result_data[0][1]['mail'][0]
                    mails.append(mail)
                    users.append(XldapClient.respuser)
                else:
                    print "[ERROR] User %s does not exist.\n" %XldapClient.respuser
                return mails, users
            elif XldapClient.node == 'egroup':
                mails = []
                users = []
                for user in XldapClient.egroup_users:
                    searchFilter = "(&(objectClass=user)(|(cn=%s)))" % user
                    ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                    result_type, result_data = self.connection.result(ldap_result_id, 0)
                    if result_data:
                        mail = result_data[0][1]['mail'][0]
                        mails.append(mail)
                        users.append(user)
                    else:
                        print "[ERROR] User %s does not exist.\n" %user
                return mails, users

        except AttributeError, error_message:
            print error_message


    def mapdevice(self):
        """
        Guess where this device should be registered according to physicalDeliveryOfficeName
        of main user(s) and repsonsible user(s)
        """
        buildings = []
        try:
            retrieveAttributes = ['physicalDeliveryOfficeName']
            if XldapClient.node == 'egroup':
                for user in XldapClient.egroup_users:
                    searchFilter = "(&(objectClass=user)(|(cn=%s)))" % user
                    ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                    result_type, result_data = self.connection.result(ldap_result_id, 0)
                    if result_data:
                        try:
                            building = result_data[0][1]['physicalDeliveryOfficeName'][0].split(' ')[0]
                        except KeyError:
                            building = ''
                        if building:
                            buildings.append(building)
            elif XldapClient.node == 'user':
                searchFilter = "(&(objectClass=user)(|(cn=%s)))" % XldapClient.respuser
                ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                result_type, result_data = self.connection.result(ldap_result_id, 0)
                if result_data:
                    try:
                        building = result_data[0][1]['physicalDeliveryOfficeName'][0].split(' ')[0]
                    except KeyError:
                        building = ''
                    if building:
                        buildings.append(building)
            if XldapClient.manager_node == 'egroup':
                for user in XldapClient.egroup_users:
                    searchFilter = "(&(objectClass=user)(|(cn=%s)))" % user
                    ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                    result_type, result_data = self.connection.result(ldap_result_id, 0)
                    if result_data:
                        try:
                            building = result_data[0][1]['physicalDeliveryOfficeName'][0].split(' ')[0]
                        except KeyError:
                            building = ''
                        if building:
                            buildings.append(building)
            elif XldapClient.manager_node == 'user':
                searchFilter = "(&(objectClass=user)(|(cn=%s)))" % XldapClient.mainuser
                ldap_result_id = self.connection.search(self.baseDN, self.searchScope, searchFilter, retrieveAttributes)
                result_type, result_data = self.connection.result(ldap_result_id, 0)
                if result_data:
                    try:
                        building = result_data[0][1]['physicalDeliveryOfficeName'][0].split(' ')[0]
                    except KeyError:
                        building = ''
                    if building:
                        buildings.append(building)
            buildings = list(set(buildings))
            return buildings

        except AttributeError, error_message:
            print error_message
